@@ -190,6 +190,8 @@ int receive_fam_notifications(FAMConnection *c, int time_limit)
 	struct stat altered_statbuf;
 	time_t ending_time;
 	filenode_type *filenode;
+	char actionname[32];
+	memset(actionname,0,sizeof(actionname));
 	ending_time = time(NULL) + time_limit;
 
 	while (ending_time >= time(NULL))
@@ -242,7 +244,14 @@ int receive_fam_notifications(FAMConnection *c, int time_limit)
 				((arguments.showchanged) && (e.code == FAMChanged) && (arguments.filechangetimeout < 0) && (!(S_ISDIR(altered_statbuf.st_mode)))))
 		{
 			//this is for when we don't have to do the timeout waiting for the file to stop changing.
-			printf("%s\n",altered_filename);
+			if (arguments.showaction)
+			{
+				if (e.code == FAMChanged)
+					snprintf(actionname,sizeof(actionname),"changed ");
+				else
+					snprintf(actionname,sizeof(actionname),"created ");
+			}
+			printf("%s%s\n",actionname,altered_filename);
 		}
 		else if (((arguments.showchanged) && (e.code == FAMChanged) && (!(S_ISDIR(altered_statbuf.st_mode)))) ||
 					((arguments.showcreated) && (e.code == FAMCreated) && (!(S_ISDIR(altered_statbuf.st_mode)))))
@@ -275,15 +284,21 @@ int receive_fam_notifications(FAMConnection *c, int time_limit)
 		}
 		else if ((arguments.showdeleted) && (e.code == FAMDeleted) && (!(S_ISDIR(altered_statbuf.st_mode))))
 		{
-			printf("%s\n",altered_filename);
+			if (arguments.showaction)
+				snprintf(actionname,sizeof(actionname),"deleted ");
+			printf("%s%s\n",actionname,altered_filename);
 		}
 		else if ((arguments.showexecuting) && (e.code == FAMStartExecuting) && (!(S_ISDIR(altered_statbuf.st_mode))))
 		{
-			printf("%s\n",altered_filename);
+			if (arguments.showaction)
+				snprintf(actionname,sizeof(actionname),"started-executing ");
+			printf("%s%s\n",actionname,altered_filename);
 		}
 		else if ((arguments.showfinishedexecuting) && (e.code == FAMStopExecuting) && (!(S_ISDIR(altered_statbuf.st_mode))))
 		{
-			printf("%s\n",altered_filename);
+			if (arguments.showaction)
+				snprintf(actionname,sizeof(actionname),"finished-executing ");
+			printf("%s%s\n",actionname,altered_filename);
 		}
 	}
 
@@ -379,6 +394,7 @@ void monitor_file(FAMConnection *c,char *buffer)
 			else
 			{
 				fprintf(stderr,"Error determining current working directory.\n");
+				perror("getcwd failed:");
 				raise(SIGINT);
 				exit(1);
 			}
@@ -639,6 +655,7 @@ const char filelist_option_explanation[]="(Default) Monitor the list of filename
 const char hashisnotcomment_option_explanation[]="Don't ignore lines in FILE starting with '#'";
 const char filechangetimeout_option_explanation[]="Assume changed after N secs of no writes. (Def=2)";
 const char onefilesystem_option_explanation[]="Don't monitor files on other filesystems";
+const char showaction_option_explanation[]="Report action when reporting altered files";
 #ifdef HAVE_ARGP_H
 #include <argp.h>
 
@@ -656,6 +673,7 @@ static struct argp_option options[] = {
 	{"hash-is-not-comment", 'H',0,0, hashisnotcomment_option_explanation},
 	{"timeout", 't',"seconds",0, filechangetimeout_option_explanation},
 	{"one-filesystem", 'x',0,0, onefilesystem_option_explanation},
+	{"show-action", 'a',0,0, showaction_option_explanation},
 	{ 0 }
 };
 
@@ -696,6 +714,9 @@ static error_t parse_opt (int key, char *arg, struct argp_state *state) {
 		case 'H':
 			arguments->hashisnotcomment=1;
 			break;
+		case 'a':
+			arguments->showaction=1;
+			break;
 		case 'x':
 			arguments->onefilesystem=1;
 			break;
@@ -722,6 +743,7 @@ void getopt_usage()
 {
 				printf("Usage: fileschanged [OPTION...] %s\n",args_doc);
 				printf("%s\n\n",doc);
+				printf("  -a                         %s\n",showaction_option_explanation);
 				printf("  -c                         %s\n",showcreated_option_explanation);
 				printf("  -C                         %s\n",showchanged_option_explanation);
 				printf("  -d                         %s\n",showdeleted_option_explanation);
@@ -770,7 +792,7 @@ memset(&arguments,0,sizeof(arguments));
 		exit(0);
 	}
 
-	while ( (i = getopt(argc,argv,"t:cCeErdfHlLx?hV")) != -1 ) { 
+	while ( (i = getopt(argc,argv,"t:acCeErdfHlLx?hV")) != -1 ) { 
 		switch(i) { 
 			case 'c':
 				arguments.showcreated=1;
@@ -810,6 +832,9 @@ memset(&arguments,0,sizeof(arguments));
 				printf("%s\n",argp_program_version);
 				exit(1);
 				break;
+			case 'a':
+				arguments.showaction=1;
+				break;
 			case 'x':
 				arguments.onefilesystem=1;
 				break;
@@ -936,7 +961,7 @@ memset(&arguments,0,sizeof(arguments));
 	//fprintf(stderr,"the result of FAMOpen is %d\n",retval);
 	if (retval < 0)
 	{
-		fprintf(stderr,"ERROR: fam server not running.  Cannot monitor files.\n");
+		fprintf(stderr,"ERROR: Cannot contact FAM server.  Cannot monitor files.\n");
 		exit(2);
 	}
 
@@ -1015,7 +1040,10 @@ memset(&arguments,0,sizeof(arguments));
 					retval = stat(filename,&statbuf);
 					if ((retval == 0) && (statbuf.st_ctime < (time(NULL)-arguments.filechangetimeout)))
 					{
-						printf("%s\n",filename);
+						if (arguments.showaction)
+							printf("changed %s\n",filename);
+						else
+							printf("%s\n", filename);
 						//printf("removing\n");
 						if (wl[wl_index])
 							free(wl[wl_index]);
